<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * Misc includes and defines necessary for the application to run.
 */
define('ALCHITECT_VERSION', 					'1.0.0.0');
define('ALCHITECT_BASE_ROOT', 					preg_replace('/core/', '', getcwd()));
define('ALCHITECT_ROOT',						ALCHITECT_BASE_ROOT . '/core');
define('ALCHITECT_CORE_SERVER_ROOT',			ALCHITECT_ROOT . '/server');
define('ALCHITECT_CORE_SERVER_CLASSES_ROOT',	ALCHITECT_CORE_SERVER_ROOT . '/classes');
define('ALCHITECT_CORE_SERVER_VENDOR_ROOT',		ALCHITECT_CORE_SERVER_ROOT . '/vendor');
define('ALCHITECT_CORE_SERVER_EXCEPTIONS_ROOT',	ALCHITECT_CORE_SERVER_CLASSES_ROOT . '/exceptions');
define('APP_PATH',								$_SERVER['SCRIPT_NAME']);
define('BASE_PATH',								preg_replace('/(\/app.*$)/', '', $_SERVER['SCRIPT_NAME']));
define('IMAGES_DIR',                            BASE_PATH . "/images");

include_once(ALCHITECT_CORE_SERVER_EXCEPTIONS_ROOT . '/ASException.php');

/**
 * Utility class for all AS* classes.
 * 
 * The purpose of this class is to be included in app.php so that the framework
 * can be fully initialized. 
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC
 * @package AlchitectCoreServer
 * @version $Id:$
 */
class ASApplication
{
	/**
	 * The constructor.
	 *
	 * @access public
	 * @return Void
	 */
	public function __construct()
	{
	}
	
	/**
	 * Set properties of the current object for the given keys.
	 * 
	 * This method looks for keys in an array and attempts to call 
	 * a setter method for that key with the value of the array.
	 * 
	 * For example, given 
	 * 
	 * <code>array('first_name'=>'John', 'last_name'=>'Doe')</code> 
	 * 
	 * the following methods will be called:
	 * 
	 * <code>
	 * $Target->setFirstName('John');
	 * $Target->setLastName('Doe');
	 * </code>
	 * 
	 * If those setter methods aren't found then the raw properties will be used:
	 * 
	 * <code>
	 * $Target->first_name = 'John';
	 * $Target->last_name = 'Doe';
	 * </code>
	 *
	 * @access public
	 * @param Array An array of keys.  Usually represents the setter name.
	 * @param Array An array of raw data to be set.
	 * @param Object The target object 
	 * @return Void
	 */
	public function setPropertiesForKeys($keys, $raw_array, $target)
	{
		if (is_array($raw_array))
		{
			$tmp_keys = array_keys($raw_array);
			
			foreach ($tmp_keys as $key)
			{
				if (is_numeric($key))
				{
					continue;
				}
				
				if (in_array($key, $keys))
				{
					$fn = 'set' . ASString::camelize($key);

					if (is_object($raw_array[$key]))
					{
						$val = $raw_array[$key]->value();
					}
					else
					{
						$val = $raw_array[$key];
					}

					if (method_exists($target, $fn))
					{
						$target->$fn($val);
					}
					else
					{
						$target->$key = $val;
					}
				}
			}
		}
	}

	/**
	 * Get a value by its associated key.
	 *
	 * @access public
	 * @param String The key to get the value for.
	 * @return Mixed
	 */
	public function valueForKey($key)
	{
		return $this->$key;
	}
}

function __autoload($class_name)
{
	if ('AS' == substr($class_name, 0, 2))
	{
		$inc_file = ALCHITECT_CORE_SERVER_CLASSES_ROOT . "/{$class_name}.php";
	}
	else if (strstr($class_name, 'Model'))
	{
		$inc_file = ALCHITECT_BASE_ROOT . "/models/{$class_name}.php";
	}
	else if (strstr($class_name, 'Controller'))
	{
		$inc_file = ALCHITECT_BASE_ROOT . "/controllers/{$class_name}.php";
	}

	if (is_file($inc_file))
	{
		include($inc_file);
	}
	else
	{
		throw new ASException("Unable to auto-load file '{$inc_file}'");
	}
}
?>
