<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Alchisoft, LLC
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreServer
 */
/**
 * A class for handling session data.
 * 
 * <b>NOTE!!</b> One of the first things you should do after installing
 * Alchitect is to change the value of AS_SESS_KEY!
 * 
 * @author Darrell Brogdon <darrell@brogdon.net>
 * @copyright (c)2002-2009 Alchisoft, LLC.
 * @package AlchitectCoreServer
 *
 */
class ASSession
{
	/**
	 * The session key.
	 * 
	 * <b>NOTE!!</b> One of the first things you should do after installing
	 * Alchitect is to change this!
	 *
	 * @final 
	 * @var String
	 */
	const AS_SESS_KEY = 'nkNFHO7ykaBrx7+Nfz4G3XjTAAU9plXjCA9LnSeU2xU=';

	/**
	 * Get the auth key.
	 * 
	 * The auth key is used to ensure the session data hasn't been tampered 
	 * with.
	 * 
	 * @access public
	 * @return String
	 */
	public static function authKey()
	{
		return md5(
			'--' 
		  . self::AS_SESS_KEY 
		  . "--{$_SESSION['alchitect']['user_id']}--{$_SESSION['alchitect']['user_name']}"
		);		
	}
	
	/**
	 * Authenticate a user based on $_SESSION['alchitect']['user_id'], 
	 * $_SESSION['alchitect']['user_name'], 
	 * and $_SESSION['alchitect']['auth_key'].
	 * 
	 * A hash of these three values (if they're present) is calculated and 
	 * compared with the value of authKey().  If the two match then the session
	 * data hasn't been altered.
	 *
	 * @access public
	 * @return boolean
	 */
	public static function authUser()
	{
		if (
			isset($_SESSION['alchitect']['user_id']) 
		 && isset($_SESSION['alchitect']['user_name']) 
		 && isset($_SESSION['alchitect']['auth_key'])
		)
		{
			return 
			(
				$_SESSION['alchitect']['auth_key'] == self::authKey()
			  ? true 
			  : false
			);
		}

		return null;
	}

	/**
	 * Initialize the session.
	 */
	public static function init()
	{
		if (self::AS_SESS_KEY == '.:! -- I Am The Alchitect -- !:.')
		{
			throw new Exception('You must change the value of AS_SESS_KEY before continuing.');
		}
		
		if (false == isset($_SESSION['alchitect']))
		{
			$_SESSION['alchitect'] = array();
		}
		
		if (false == isset($_SESSION['alchitect']['auth_key'])
		&&  isset($_COOKIE['auth_key'])
		&&  isset($_COOKIE['user_id'])
		&&  isset($_COOKIE['user_name']))
		{
			$user_data = array
			(
				'user_id'	=> $_COOKIE['user_id']
			  , 'user_name'	=> $_COOKIE['user_name']
			);
			
			self::saveUserData($user_data);
		}
	}
	
	
	/**
	 * Unset all session data.
	 * 
	 * @access public
	 * @return Void
	 */
	public static function reset()
	{
		foreach($_SESSION['alchitect'] as $key => $value)
		{
			self::unsetValueForKey($key);
		}
		
		return self;
	}
	
	/**
	 * Save user data into the session.
	 *
	 * @access public
	 * @param Array The user data.
	 * @return Void
	 */
	public static function saveUserData($user_data)
	{
		$_SESSION['alchitect']['user_id']		= $user_data['user_id'];
		$_SESSION['alchitect']['user_name']	= $user_data['user_name'];
		$_SESSION['alchitect']['auth_key']		= md5
		(
			self::AS_SESS_KEY
		  . '--'
		  . $user_data['user_id']
		  . '--'
		  . $user_data['user_name']
		);
	}
	
	/**
	 * Helper function for setting data in the session for a given key.
	 * 
	 * @access public
	 * @param Mixed The name of value to set.
	 * @param Mixed The value.
	 * @return Void
	 */
	public static function setValueForKey($key, $value)
	{
		$_SESSION['alchitect'][$key] = $value;
	}
	
	/**
	 * Unset a value in the session for a given key.
	 * 
	 * @access public
	 * @static 
	 * @param Mixed The name of the value to unset.
	 * @return Void
	 */
	public static function unsetValueForKey($key)
	{
		unset($_SESSION['alchitect'][$key]);
		
		// Must use setcookie() here as trying to do unset($_COOKIE[$key]) 
		// doesn't actually remove the cookie from the browser.
		setcookie($key, "", time() - 3600);
		
		// Just to be sure :)
		unset($_COOKIE[$key]);
	}
	
	/**
	 * Get the value in the session for a given key.
	 * 
	 * @access public
	 * @static 
	 * @param Mixed The key to fetch the value for.
	 * @return Mixed
	 */
	public static function valueForKey($key)
	{
		return (isset($_SESSION['alchitect'][$key]) ? $_SESSION['alchitect'][$key] : null);
	}
}
?>