/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview An exception for use by ASAuthentication.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class An exception for use by ASAuthentication.
 *
 * @constructor
 * @param The exception message.
 * @param The exception code.
 * @returns An instance of ASAuthenticationException.
 * @type Object
 */
function ASAuthenticationException(code, message)
{
	this.setCode(code);
	this.setMessage(message);
}

// Extend ASException
ASAuthenticationException.prototype = new ASException();

/**
 * A property representing a Missing Username exception.
 * 
 * @type Integer
 */
ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_USERNAME = 1020;

/**
 * A property representing a Missing Password exception.
 * 
 * @type Integer
 */
ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_PASSWORD = 1030;

/**
 * A property representing a Missing Auth Hash exception.
 * 
 * @type Integer
 */
ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_AUTH_HASH = 1040;

ASException.messages.push(ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_USERNAME	= 'Missing username');
ASException.messages.push(ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_PASSWORD	= 'Missing password');
ASException.messages.push(ASAuthenticationException.AS_AUTH_EXCEPTION_MISSING_AUTH_HASH	= 'Missing authentication hash');

/**
 * Convert the exception to a string.
 *
 * @returns The exception string.
 * @type String
 */
ASAuthenticationException.prototype.toString = function()
{
	return "ASAuthenticationException (" + this.code + ") " + this.message;
}
