/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview A general Alchitect exception.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A general Alchitect exception.
 *
 * If the message isn't provided then a lookup is made based
 * on the code.
 *
 * @constructor
 * @param The exception message.
 * @param The exception code.
 * @returns An instance of ASException.
 * @type Object
 */
function ASException(code, message)
{
	this.setCode(code);
	
	if (null == message)
	{
		this.setMessage(this.exceptions[code]);
	}
	else
	{
		this.setMessage(message);
	}
}

// Extend Error.
ASException.prototype = new Error();

/**
 * A property representing Unknown exception.
 * 
 * @type Integer
 */
ASException.AS_EXCEPTION_UNKNOWN = 9999;

/**
 * A property representing a missing ID exception.
 * 
 * @type Integer
 */
ASException.AS_EXCEPTION_MISSING_ID = 1000;

/**
 * An array of exception codes and their corresponding messages.
 * This is used during the lookup when the exception message isn't provided
 * to the constructor.
 */
ASException.messages = [
	ASException.AS_EXCEPTION_UNKNOWN				= 'Unknown Exception'
  , ASException.AS_EXCEPTION_MISSING_ID				= 'Missing ID'
];

/**
 * Get the message for a given code.
 *
 * @param The exception code.
 * @returns The message.
 * @type String
 */
ASException.prototype.messageForExceptionCode = function(code)
{
	return this.messages[code];
}

/**
 * Set the exception code.
 *
 * @param The code.
 * @returns Void.
 */
ASException.prototype.setCode = function(code)
{
	this.code = code;
}

/**
 * Set the exception message.
 *
 * @param The message.
 * @returns Void.
 */
ASException.prototype.setMessage = function(message)
{
	this.message = message;
}

/**
 * Convert the exception to a string.
 *
 * @returns The exception string.
 * @type String
 */
ASException.prototype.toString = function()
{
	return "ASException (" + this.code + ") " + this.message;
}
