/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview A parent Object class that most all Alchitect classes
 * should extend.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A parent Object class
 * @constructor
 * @returns An instance of ASObject.
 * @type Object
 */
function ASObject()
{
	/**
	 * The ID.
	 * 
	 * @type Integer
	 */
	this.id = 0;
}

// Extend JavaScript Object
ASObject.prototype = new Object();

/**
 * Get the ID.
 *
 * @returns The ID.
 * @type Integer
 */
ASObject.prototype.id = function()
{
	return this.id;
}

/**
 * Set the ID
 *
 * @param The ID.
 * @returns Void.
 * @type Void
 */
ASObject.prototype.setID = function(id)
{
	this.id = id;
}

/**
 * Set the value of a property for a given key.
 *
 * This method looks for a setter method named "set<key>" (i.e. if key='foo' then
 * use 'setfoo()') and will use that to set the value if found.  Otherwise it 
 * sets the value of the property directly.
 *
 * @param The key.
 * @param The value.
 * @returns The instance of ASObject.
 * @type Object
 */
ASObject.prototype.setValueForKey = function(key, value)
{
	var method = "set" + key;

	if (this.method)
	{
		this.method(value);
	}
	else
	{
		this.key = value;
	}

	return this;
}

/**
 * Get the value of a property based on a given key which will usually represent
 * the property name.
 *
 * This method looks for a getter method named "get<key>" (i.e. if (key='foo' then
 * use 'getfoo()') and will use that method to get the value if found. Otherwise it
 * gets the value of the property directly.
 *
 * @param The key.
 * @returns The value.
 * @type Mixed
 */
ASObject.prototype.valueForKey = function(key)
{
	var value	= null;
	var method	= "get" + key;

	if (this.method)
	{
		value = this.method();
	}
	else
	{
		value = this.key;
	}

	return value;
}
