/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectCoreUI
 */
/** 
 * @fileoverview For every new Unit Test you create you must add it to 
 * the 'unit_tests' array in order for it to be used.
 * @author Darrell Brogdon &lt;{@link http://mailto:darrell@brogdon.net darrell@brogdon.net}&gt;
 * @copyright 2002-2009 Darrell Brogdon - All Rights Reserved
 * @version 1.0.0.0 
 */
/**
 * @class A class for managing multiple Unit Tests.
 * @constructor
 * @param Whether to log the results of the individual tests
 * to the console.
 * @returns An instance of ASUnitTests.
 * @type Object
 */
function ASUnitTests(log_to_console)
{
	/**
	 * An array of Unit Tests that need to be executed.
	 *
	 * @type Array
	 */
	this.unit_tests =
	{
		'UnitTestTest':	ASUnitTestTest
	  , 'ObjectTest':	ASObjectTest
	};

	/**
	 * Whether to log testing info to the console.
	 *
	 * @type Boolean
	 */
	this.log_to_console = log_to_console || false;
	
	/**
	 * An array of tests that have been executed.
	 *
	 * @type Array
	 */
	this.tests = [];
	
	/**
	 * A count of how many total tests have passed.
	 *
	 * @type Integer
	 */
	this.passed			= 0;
	
	/**
	 * A count of how many total tests have failed.
	 *
	 * @type Integer
	 */
	this.failed			= 0;
	
	/**
	 * A count of how many total tests have been executed.
	 *
	 * @type Integer
	 */
	this.total			= 0;
}

/**
 * Get all the Unit Tests.
 *
 * @returns An array of Unit Tests.
 * @type Array
 */
ASUnitTests.prototype.unitTests = function()
{
	return this.unit_tests;
}

/**
 * Execute all the Unit Tests defined in the 'unit_tests' array.
 *
 * @returns Void.
 * @type Void
 */
ASUnitTests.prototype.run = function()
{
	var x = null;

	for (x in this.unit_tests)
	{
		if (this.log_to_console)
		{
			console.log('RUNNING: %s', this.unit_tests[x]);
		}

		var _test = new this.unit_tests[x](this.log_to_console);

		this.passed += _test.passed;
		this.failed += _test.failed;
		this.total++;

		this.tests.push(_test);
	}
}
