<?php
/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 */
define('BASE_PATH', str_replace('install.php', '', $_SERVER['SCRIPT_NAME']));

$short_open_tag_on			= ini_get('short_open_tag');
$already_installed			= false;
$successful_installation	= false;
$validation_errors			= array();
$permissions_errors			= array();
$loaded_modules				= get_loaded_extensions();
$required_modules			= array('zip', 'PDO', 'SimpleXML', 'pdo_mysql', 'json', 'filter', 'mhash');
$missing_modules			= array();

foreach ($required_modules as $mod)
{
	if (false == in_array($mod, $loaded_modules))
	{
		$missing_modules[] = $mod;
	}
}

if ('nt authority\iusr' == trim(`whoami`) && false == is_writable('.') || false == is_writable('pages/template.html.php'))
{
	$permissions_errors[] = "The installer cannot be run yet. You'll need to add the '"
						  . "<code style=\"font-size:1.25em;\">" . trim(`whoami`) . "</code>"
						  . "' user to the list of users who can Modify the <code style=\"font-size:1.25em;\">wwwroot</code> directory. "
						  . "When you've done that simply refresh this page and proceed with the install."
						  . "<p>You can find more help at the <a href=\"http://alchitect.com/app/documentation/install#IIS7\">Alchitect support page</a>.</p>";
}
else if (false == is_writable('.') || false == is_writable('pages/template.html.php'))
{
	$permissions_errors[] = "The installer cannot be run yet. You must first run the pre_install.sh "
						  . "shell script found in the 'bin' directory.<br /><br />"
						  . "When prompted for the name of the web server group you will need to enter: "
						  . "<pre style=\"font-size:1.25em;\">\t" . trim(`whoami`) . "</pre>";
}

if (file_exists('config.ini'))
{
	$already_installed = true;
}

if (isset($_POST['form_action']) && 'install_alchitect' == $_POST['form_action'])
{
	$site_name	= filter_var($_POST['site_name'],		FILTER_SANITIZE_STRING);
	$admin_user	= filter_var($_POST['admin_user'],		FILTER_SANITIZE_STRING);
	$admin_pass	= filter_var($_POST['admin_password'],	FILTER_SANITIZE_STRING);
	$db_type	= 'MySQL'; //strtolower(filter_var($_POST['db_type'], FILTER_SANITIZE_STRING));
	$db_host	= filter_var($_POST['db_host'],			FILTER_SANITIZE_STRING);
	$db_name	= filter_var($_POST['db_name'],			FILTER_SANITIZE_STRING);
	$db_user	= filter_var($_POST['db_user'],			FILTER_SANITIZE_STRING);
	$db_pass	= filter_var($_POST['db_pass'],			FILTER_SANITIZE_STRING);
	$mysql_conn	= test_mysql_connection
				(
					strtolower($db_type)
				  , $db_host
				  , $db_name
				  , $db_user
				  , $db_pass
				  , $admin_user
				  , $admin_pass
				);
	
	if (true !== $mysql_conn)
	{
		$validation_errors[] = $mysql_conn;
	}
	else
	{
		update_html_template($admin_user, $site_name);
	}
	
	if (false == sizeof($validation_errors))
	{
		$successful_installation = true;
	}
}
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
	<meta name="author" content="Darrell Brogdon" />
	<meta name="description" content="Alchitect installation page" />
	<meta name="keywords" content="alchitect,alchisoft" />
	<style type="text/css" media="screen">
	/***** Global Settings *****/

	html, body {
	border:0;
	margin:0;
	padding:0;
	}

	body {
	font:100%/1.25 Arial, Helvetica, sans-serif;
	}

	/***** Headings *****/

	h1, h2, h3, h4, h5, h6 {
	margin:0;
	padding:0;
	font-weight:normal;
	}

	h1 {
	padding:30px 0 25px 0;
	letter-spacing:-1px;
	font-size:2em;
	}

	h2 {
	padding:20px 0;
	letter-spacing:-1px;
	font-size:1.5em;
	}

	h3 {
	font-size:1em;
	font-weight:bold;
	}

	/***** Common Formatting *****/

	p, ul, ol {
	margin:0;
	padding:0 0 1.25em 0;
	}

	ul, ol {
	padding:0 0 1.25em 2.5em;
	}

	blockquote {
	margin:1.25em;
	padding:1.25em 1.25em 0 1.25em;
	}

	small {
	font-size:0.85em;
	}

	img {
	border:0;
	}

	sup {
	position:relative;
	bottom:0.3em;
	vertical-align:baseline;
	}

	sub {
	position:relative;
	bottom:-0.2em;
	vertical-align:baseline;
	}

	acronym, abbr {
	cursor:help;
	letter-spacing:1px;
	border-bottom:1px dashed;
	}

	/***** Links *****/

	a,
	a:link,
	a:visited,
	a:hover {
	text-decoration:underline;
	}

	/***** Forms *****/

	form {
	margin:0;
	padding:0;
	display:inline;
	}

	input, select, textarea {
	font:1em Arial, Helvetica, sans-serif;
	}

	textarea {
	width:100%;
	line-height:1.25;
	}

	label {
	cursor:pointer;
	}

	/***** Tables *****/

	table {
	border:0;
	margin:0 0 1.25em 0;
	padding:0;
	}

	table tr td {
	padding:2px;
	}

	/***** Wrapper *****/

	#wrap {
	width:960px;
	margin:0 auto;
	}

	/***** Global Classes *****/

	.clear         { clear:both; }
	.float-left    { float:left; }
	.float-right   { float:right; }

	.text-left     { text-align:left; }
	.text-right    { text-align:right; }
	.text-center   { text-align:center; }
	.text-justify  { text-align:justify; }

	.bold          { font-weight:bold; }
	.italic        { font-style:italic; }
	.underline     { border-bottom:1px solid; }
	.highlight     { background:#ffc; }

	.wrap          { width:960px;margin:0 auto; }

	.img-left      { float:left;margin:4px 10px 4px 0; }
	.img-right     { float:right;margin:4px 0 4px 10px; }

	.nopadding     { padding:0; }
	.noindent      { margin-left:0;padding-left:0; }
	.nobullet      { list-style:none;list-style-image:none; }
	
	A, A:link, A:visited {
		color: #233447;
		text-decoration: none;
	}

	BODY {
		background-color: #F4F7F0;
	}

	.add_on_detail {
		background-color:white;
		border-radius:0px 0px 5px 5px;
		color:Black;
		-moz-border-radius:0px 0px 5px 5px;
		-webkit-border-bottom-left-radius:5px;
		-webkit-border-bottom-right-radius:5px;
	}

	.add_on_title {
		cursor:pointer;
		font-weight:bold;
	}

	.add_on_title_header {
		background-color: #233447;
		border-radius:5px 5px 0px 0px;
		padding: 5px;
		-moz-border-radius:5px 5px 0px 0px;
		-webkit-border-top-left-radius:5px;
		-webkit-border-top-right-radius:5px;
	}

	.copyright {
		color:Gray;
		font-size:0.50em;
		text-align:center;
	}

	.default {
		background-color:#b6bdc9;
		border:3px solid gray;
		border-radius:15px;
		color:#FFF;
		padding:10px;
		text-align:left;
		width:600px;
		-moz-border-radius:15px;
		-webkit-border-radius:15px;
	}
	
	.error {
		background-color:pink;
		border:1px solid red;
		color:maroon;
		padding:10px;
		margin-bottom:10px;
		width:500px;
	}
	
	.gray_input_text {
		color: Gray;
	}
	
	.input_error {
		background-color:pink;
		border:1px solid red;		
	}
	
	.lbl {
		float:left;
		padding-right:10px;
		text-align:right;
		width:200px;
	}
	
	.row {
		margin:5px;
	}
	</style>
	<script src="<?=BASE_PATH?>core/ui/vendor/jquery/jquery-1.3.2.js" type="text/javascript" charset="utf-8"></script>
	<script type="text/javascript" charset="utf-8">
		$(function(){
			$('#site_name')
				.focus()
				.blur(function(evt){
					$('#db_name').val($(this).val().toLowerCase().replace(/ /, '_'));
				});
			
			$('#admin_user').focus(function(evt){
				$(this).removeClass('gray_input_text');
				
				if ('user@domain.tld' == $(this).val())
				{
					$(this).val('');
				}
			});
			
			$('#db_host').focus(function(evt){
				$(this).removeClass('gray_input_text');
			});
			
			$('.validate_text').focus(function(evt){
				$(this).removeClass('input_error');
			});
			
			$('#btn_install').click(function(evt){
				if (true === validateText() && validatePasswords())
				{
					$('#frm_install').submit();
				}
			});
		});
		
		function validatePasswords()
		{
			var msg			= 'The passwords do not match';
			var validated	= true;
			var pass		= $('#admin_password');
			var confirm		= $('#confirm_admin_password');
			
			if (pass.val() !== confirm.val())
			{
				$('.error').html(msg).show();
				pass.addClass('input_error');
				confirm.addClass('input_error');
				validated = false;
			}
			
			return validated;
		}
		
		function validateText()
		{
			var msg			= 'The highlighted fields must not be empty!';
			var validated	= true;
			
			$.each(
				$('.validate_text')
			  , function(idx, item)
				{
					if ('' == $(item).val())
					{
						validated = false;
						$(item).addClass('input_error');
					}
				}
			);
			
			if (false == validated)
			{
				$('.error').html(msg).show();
			}
			
			return validated;
		}
	</script>
	<title>Alchitect Installer</title>
</head>
<body>

<form method="post" id="frm_install" action="<?=filter_var($_SERVER['PHP_SELF'], FILTER_SANITIZE_STRING)?>">
<div align="center">
    <h2 style="color:#233447">Alchitect Installer</h2>

	<div id="error" class="error" style="display:none;" align="center"></div>

	<div class="default">
<?php if (false == $short_open_tag_on): ?>
		<p>
			Alchitect requires that you <a href="http://www.php.net/manual/en/ini.core.php#ini.short-open-tag">enable short_open_tag</a> in your php.ini file.
		</p>
<?php elseif (true === $already_installed): ?>
		<p>
			Alchitect has already been installed. For security purposes you should remove this
			installer.
		</p>
<?php elseif (true === $successful_installation):?>
		<h2 align="center" style="color:#233447;">Success!</h2>
		<p>Alchitect has now been installed.</p>
	
		<p>For added security, you should remove the installer file from:</p>
	
		<p>
			<code><?=getcwd()?>/install.php</code>
		</p>
		
		<p>
			The next step is to visit the Alchitect Admin page located at 
			<a href="<?='http://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF'])?>app/admin"><?='http://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF'])?>app/admin</a>.
		</p>
<?php elseif (sizeof($validation_errors)):?>
		<h2 align="center" style="color:Red;">ERROR</h2>
		<p>The following errors occurred during installation:</p>

		<ul>
		<?php foreach($validation_errors as $err): ?>
			<li><?=$err?></li>
		<?php endforeach; ?>
		</ul>	
<?php elseif (sizeof($missing_modules)): ?>
		<p>The installer could not find the following PHP module(s):</p>

		<ul>
		<?php foreach($missing_modules as $mod): ?>
			<li><a href="http://php.net/<?=$mod?>"><?=$mod?></a></li>
		<?php endforeach; ?>
		</ul>

		<p>Please install any modules that might be missing and try this page again. </p>
<?php elseif (sizeof($permissions_errors)): ?>
		<p><?=$permissions_errors[0]?></p>
<?php else: ?>
		<p>
		Welcome to the Alchitect installation page.  Installation is as easy as following the steps
		listed below.
		</p>
		
		<div class="add_on_title_header">
			Step 1: Site Information
		</div>
		<div class="add_on_detail" style="padding:5px;">
			<div class="row">
				<div class="lbl">
					<label for="site_name">Site Name</label>
				</div>
				<div>
					<input type="text" name="site_name" id="site_name" class="validate_text" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="admin_user">Admin Email</label>
				</div>
				<div>
					<input type="text" name="admin_user" id="admin_user" class="validate_text gray_input_text" value="user@domain.tld" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="admin_password">Admin Password</label>
				</div>
				<div>
					<input type="password" name="admin_password" id="admin_password" class="validate_text" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="confirm_admin_password">Confirm Admin Password</label>
				</div>
				<div>
					<input type="password" name="confirm_admin_password" id="confirm_admin_password" class="validate_text" />
				</div>
			</div>
		</div>
		
		<br />
		
		<div class="add_on_title_header">
			Step 2: Database
		</div>
		<div class="add_on_detail" style="padding:5px;">
			<div class="row">
				<div class="lbl">
					<label for="db_host">Host Name</label>
				</div>
				<div>
					<input type="text" name="db_host" value="localhost" id="db_host" class="validate_text gray_input_text" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="db_type">Type</label>
				</div>
				<div>
					MySQL
					<!--
					<select name="db_type" id="db_type">
						<option value="MySQL">MySQL</option>
						<option value="PgSQL">PostgreSQL</option>
					</select>
					-->
				</div>
			</div>

			<div class="row">
				<div class="lbl">
					<label for="db_name">Name</label>
				</div>
				<div>
					<input type="text" name="db_name" id="db_name" class="validate_text" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="db_user">User Name</label>
				</div>
				<div>
					<input type="text" name="db_user" id="db_user" class="validate_text" />
				</div>
			</div>
			
			<div class="row">
				<div class="lbl">
					<label for="db_pass">Password</label>
				</div>
				<div>
					<input type="password" name="db_pass" id="db_pass" />
				</div>
			</div>
		</div>

		<br />

		<div class="add_on_title_header">
			Step 3: Install
		</div>
		<div class="add_on_detail" style="padding:5px;">
			<div class="row">
				<div class="lbl">&nbsp;</div>
				<div>
					<input type="hidden" name="form_action" value="install_alchitect" id="form_action" />
					<input type="button" id="btn_install" value="Install Alchitect!" />
				</div>
			</div>
		</div>
<?php endif; ?>
	</div>
	
	<br />
    
    <a href="http://alchitect.com"><img src="http://alchitect.com/images/Alchitect_Powered_By_135x42.png" width="135" height="42" alt="Powered By Alchitect" border="0" /></a>

	<br />
	
	<div class="copyright">
		&copy;2002-<?=date('Y')?> Alchisoft, LLC
	</div>
</div>
</form>

</body>
<?php
function set_session_key()
{
	$as_session = file_get_contents('core/server/classes/ASSession.php');
	$old_session_key = "const AS_SESS_KEY = '.:! -- I Am The Alchitect -- !:.";
	$new_session_key = "const AS_SESS_KEY = '" . base64_encode(mhash(MHASH_SHA256, mt_rand(mt_getrandmax(), mt_getrandmax()) . microtime()));

	$as_session = str_replace($old_session_key, $new_session_key, $as_session);

	file_put_contents('core/server/classes/ASSession.php', $as_session);
}

function create_config_file($db_type, $db_host, $db_name, $db_user, $db_pass)
{
	$template_vars	= array('db_type', 'db_host', 'db_name', 'db_user', 'db_pass');
	$config_ini		= file_get_contents('pages/admin/templates/config.ini');
	
	foreach ($template_vars as $var)
	{
		$tpl_var	= '@@' . strtoupper($var) . '@@';
		$config_ini	= str_replace($tpl_var, ${$var}, $config_ini);
	}
	
	file_put_contents('config.ini', $config_ini);
}

function update_html_template($author_name, $site_title, $site_description=null, $site_keywords=null)
{
	$alchitect_generator	= 'Alchitect v1.0';
	$template_vars			= array
	(
		'author_name'
	  , 'site_title'
	  , 'site_description'
	  , 'site_keywords'
	  , 'alchitect_generator'
	);
	$template_html			= file_get_contents('pages/template.html.php');
	
	foreach ($template_vars as $var)
	{
		$tpl_var = '@@' . strtoupper($var) . '@@';
		$template_html = str_replace($tpl_var, ${$var}, $template_html);
	}
	
	file_put_contents('pages/template.html.php', $template_html);
}

function test_mysql_connection($type, $host, $name, $user, $pass, $admin_user, $admin_pass)
{
	try
	{
		$dbh		= new PDO("{$type}:host={$host};dbname=mysql", $user, $pass);
		$stmt		= $dbh->query("SHOW DATABASES");
		$res		= $stmt->fetchAll();
		$created	= false;
		
		foreach ($res as $db)
		{
			if ($db['Database'] == $name)
			{
				$created = true;
				break;
			}
		}
		
		if (false === $created)
		{
			$stmt = $dbh->query("CREATE DATABASE {$name}");
		
			if ('00000' != $dbh->errorCode())
			{
				$msg = $dbh->errorInfo();
				throw new Exception($msg[2], $msg[1]);
			}
		}
		
		$dbh = new PDO("{$type}:host={$host};dbname={$name}", $user, $pass);
		$sql = "
CREATE TABLE user_account (
	id               varchar(36)         NOT NULL DEFAULT '00000000-0000-0000-0000-000000000000'
  , email_address    varchar(255)        NOT NULL
  , password         varchar(32)         NOT NULL
  , is_admin         tinyint(1) unsigned NOT NULL DEfAULT 0
  , created_on       datetime            NOT NULL
  , last_modified_on timestamp           NOT NULL DEFAULT CURRENT_TIMESTAMP
  , last_modified_by varchar(37)         NOT NULL
  , PRIMARY KEY (id)
)";
		$stmt = $dbh->query($sql);
		
		if ('00000' != $dbh->errorCode())
		{
			$msg = $dbh->errorInfo();
			throw new Exception($msg[2], $msg[1]);
		}
		
		$sql = "
INSERT INTO user_account (
	id
  , email_address
  , password
  , is_admin
  , created_on
) VALUES (
	UUID()
  , '{$admin_user}'
  , '" . md5($admin_pass) . "'
  , '1'
  , '" . date('Y-m-d H:i:s', time()) . "'
)";

		$stmt = $dbh->query($sql);
		
		if ('00000' != $dbh->errorCode())
		{
			$msg = $dbh->errorInfo();
			throw new Exception($msg[2], $msg[1]);
		}
		
		create_config_file($type, $host, $name, $user, $pass);
		set_session_key();
		
		return true;
	}
	catch (Exception $ex)
	{
		return $ex->getMessage();
	}
}
?>
