/**
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is Alchitect Core Server.
 * 
 * The Initial Developer of the Original Code is Darrell Brogdon.
 * Portions created by Darrell Brogdon are Copyright (C) 2002-2009 Darrell Brogdon.
 * All Rights Reserved.
 * 
 * Contributor(s): .
 * 
 * @package AlchitectUserAccountModule
 */
$(function()
{
	AS_FORM.registerHook(verifyPasswordConfirmationHook);
	AS_FORM.registerFormSubmittedCallback(change_password_form_submitted);
});

/**
 * Callback function to be executed after the form has been submitted, the XHR
 * request has been made, and data is ready to be acted upon.
 *
 * @param Object The XHR response data object
 * @param String The XHR response string
 */
function change_password_form_submitted(data, status)
{
	console.log('DATA: %o', data);
	console.log('STATUS: %s', status);
	
	if (false == data.value)
	{
		$('#error').html('Failed to update user account - Please try again').show();
	}
	else if ('Exception' == data.type)
	{
		$('#error').html('ERROR (' + data.code + '): ' + data.value).show();
	}
	else
	{
		$('#confirmation').html('User account information has been updated').show();

		console.log('FORM ACTION: %s', $('#form_action').val());

		if ('delete' == $('#form_action').val())
		{
			window.history.back();
		}
	}
}

/**
 * Confirm the user really wants to delete the specified user record.
 */
function confirmDelete()
{
	if (confirm('Are you sure you want to delete this User Account?\n\nThis action CANNOT be undone!'))
	{
		$('#form_action').val('delete');
		
		return true;
	}

	return false;
}

/**
 * Reset the password confirmation error if any and hide it.
 */
function resetPasswordConfirmation()
{
	$('#confirm_password').parent().removeClass('error_input');
	$('#password_confirmation_row').hide();
}

/**
 * Show the password confirmation row. 
 */
function showPasswordConfirmationRow()
{
	$('#password_confirmation_row').show();
}

/**
 * Toggle the password confirmation row.
 */
function togglePasswordConfirmationRow()
{
	if ('' == $('#confirm_password').val() && '' == $('#password').val())
	{
		$('#password_confirmation_row').hide();
		$('#password').parent().removeClass('error_input');
		$('#confirm_password').parent().removeClass('error_input');
	}
}

/**
 * Verify the the password confirmation value matches the password value.
 *
 * @param Object The form object.
 */
function verifyPasswordConfirmationHook(frm)
{
	if ('delete' == $('#form_action').val())
	{
		return true;
	}
	
	if ($('#confirm_password').val() != $('#password').val()
	|| '' == $('#confirm_password').val()
	|| '' == $('#password').val())
	{
		if ('' == $('#password').val())
		{
			$('#password').parent().addClass('error_input');
		}
		
		$('#confirm_password').parent().addClass('error_input');
		$('#confirm_password').focus().select();
		
		return false;
	}

	$('#password').parent().removeClass('error_input');
	$('#confirm_password').parent().removeClass('error_input');

	return true;
}